class CorpusDisplay extends RessourceDisplay {
  constructor(displayElement) {
    super(displayElement);
    this.corpusId = displayElement.dataset.corpusId;
    for (let exportCorpusTriggerElement of this.displayElement.querySelectorAll('.export-corpus-trigger')) {
      exportCorpusTriggerElement.addEventListener('click', () => this.requestCorpusExport());
    }
    app.socket.on(`export_corpus_${this.corpusId}`, () => this.downloadCorpus());
  }

  init(user) {
    let corpus;
    corpus = user.corpora[this.corpusId];
    this.setCreationDate(corpus.creation_date);
    this.setDescription(corpus.description);
    this.setLastEditedDate(corpus.last_edited_date);
    this.setStatus(corpus.status);
    this.setTitle(corpus.title);
    this.setTokenRatio(corpus.num_tokens, corpus.max_num_tokens);
  }

  patch(patch) {
    let re, filteredPatch;
    re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}`);
    filteredPatch = patch.filter(operation => re.test(operation.path));
    for (let operation of filteredPatch) {
      switch(operation.op) {
        case 'replace':
          re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/last_edited_date$`);
          if (re.test(operation.path)) {this.setLastEditedDate(operation.value); break;}
          re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/status$`);
          if (re.test(operation.path)) {this.status$(operation.value); break;}
          break;
        default:
          break;
      }
    }
  }

  requestCorpusExport() {
    app.socket.emit('export_corpus', app.users[this.userId].corpora[this.corpusId]);
    app.flash('Preparing your corpus export...', 'corpus');
    for (let exportCorpusTriggerElement of this.displayElement.querySelectorAll('.export-corpus-trigger')) {exportCorpusTriggerElement.classList.toggle('disabled', true);}
  }

  downloadCorpus() {
    let downloadButton;
    app.flash('Corpus download is ready!', 'corpus');
    for (let exportCorpusTriggerElement of this.displayElement.querySelectorAll('.export-corpus-trigger')) {exportCorpusTriggerElement.classList.toggle('disabled', false);}
    // Little trick to call the download view after ziping has finished
    downloadButton = document.createElement('a');
    downloadButton.href = `/corpora/${app.users[this.userId].corpora[this.corpusId]}/download`;
    downloadButton.click();
  }

  setTitle(title) {
    for (let element of this.displayElement.querySelectorAll('.corpus-title')) {this.setElement(element, title);}
  }

  setTokenRatio(numTokens, maxNumTokens) {
    for (let element of this.displayElement.querySelectorAll('.corpus-token-ratio')) {this.setElement(element, `${numTokens}/${maxNumTokens}`);}
  }

  setDescription(description) {
    for (let element of this.displayElement.querySelectorAll('.corpus-description')) {this.setElement(element, description);}
  }

  setStatus(status) {
    for (let element of this.displayElement.querySelectorAll('.analyse-corpus-trigger')) {
      if (['analysing', 'prepared', 'start analysis'].includes(status)) {
        element.classList.remove('disabled');
      } else {
        element.classList.add('disabled');
      }
    }
    for (let element of this.displayElement.querySelectorAll('.build-corpus-trigger')) {
      if (status === 'unprepared' && Object.values(app.users[this.userId].corpora[this.corpusId].files).length > 0) {
        element.classList.remove('disabled');
      } else {
        element.classList.add('disabled');
      }
    }
    for (let element of this.displayElement.querySelectorAll('.corpus-status')) {this.setElement(element, status);}
    for (let exportCorpusTriggerElement of this.displayElement.querySelectorAll('.export-corpus-trigger')) {
      exportCorpusTriggerElement.classList.toggle('disabled', !['prepared', 'start analysis', 'stop analysis'].includes(status));
    }
    for (let element of this.displayElement.querySelectorAll('.status')) {element.dataset.status = status;}
    for (let element of this.displayElement.querySelectorAll('.status-spinner')) {
      if (['submitted', 'queued', 'running', 'canceling', 'start analysis', 'stop analysis'].includes(status)) {
        element.classList.remove('hide');
      } else {
        element.classList.add('hide');
      }
    }
  }

  setCreationDate(creationDate) {
    for (let element of this.displayElement.querySelectorAll('.corpus-creation-date')) {
      this.setElement(element, creationDate.toLocaleString("en-US"));
    }
  }

  setLastEditedDate(lastEditedDate) {
    for (let element of this.displayElement.querySelectorAll('.corpus-end-date')) {
      this.setElement(element, lastEditedDate.toLocaleString("en-US"));
    }
  }
}
