class CorpusFileList extends RessourceList {
  constructor(listElement, options = {}) {
    super(listElement, {...CorpusFileList.options, ...options});
    this.corpusId = listElement.dataset.corpusId;
  }

  init(user) {
    this._init(user.corpora[this.corpusId].files);
  }

  onclick(event) {
    let corpusFileElement = event.target.closest('tr[data-id]');
    if (corpusFileElement === null) {throw 'Could not locate corpus file element';}
    let corpusFileId = corpusFileElement.dataset.id;
    let actionButtonElement = event.target.closest('.action-button[data-action]');
    let action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete':
        let deleteModalHTML = `
          <div class="modal">
            <div class="modal-content">
              <h4>Confirm corpus deletion</h4>
              <p>Do you really want to delete the corpus file <b>${app.users[this.userId].corpora[this.corpusId].files[corpusFileId].filename}</b>? It will be permanently deleted!</p>
            </div>
            <div class="modal-footer">
              <a href="#!" class="btn modal-close waves-effect waves-light">Cancel</a>
              <a class="btn modal-close red waves-effect waves-light" href="/corpora/${this.corpusId}/files/${corpusFileId}/delete"><i class="material-icons left">delete</i>Delete</a>
            </div>
          </div>
        `.trim();
        let deleteModalParentElement = document.querySelector('#modals');
        deleteModalParentElement.insertAdjacentHTML('beforeend', deleteModalHTML);
        let deleteModalElement = deleteModalParentElement.lastChild;
        let deleteModal = M.Modal.init(deleteModalElement, {onCloseEnd: () => {deleteModal.destroy(); deleteModalElement.remove();}});
        deleteModal.open();
        break;
      case 'download':
        window.location.href = `/corpora/${this.corpusId}/files/${corpusFileId}/download`;
        break;
      case 'view':
        window.location.href = `/corpora/${this.corpusId}/files/${corpusFileId}`;
        break;
      default:
        break;
    }
  }

  usersPatchHandler(patch) {
    let re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/files/([A-Za-z0-9]*)`);
    let filteredPatch = patch.filter(operation => re.test(operation.path));
    for (let operation of filteredPatch) {
      switch(operation.op) {
        case 'add':
          re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/files/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        case 'remove':
          re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/files/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {
            [match, corpusFileId] = operation.path.match(re);
            this.remove(corpusFileId);
          }
          break;
        case 'replace':
          re = new RegExp(`^/users/${this.userId}/corpora/${this.corpusId}/files/([A-Za-z0-9]*)/(author|filename|publishing_year|title)$`);
          if (re.test(operation.path)) {
            [match, corpusFileId, valueName] = operation.path.match(re);
            this.replace(corpusFileId, valueName, operation.value);
          }
          break;
        default:
          break;
      }
    }
  }

  preprocessRessource(corpusFile) {
    return {
      id: corpusFile.id,
      author: corpusFile.author,
      creationDate: corpusFile.creation_date,
      filename: corpusFile.filename,
      publishing_year: corpusFile.publishing_year,
      title: corpusFile.title
    };
  }
}
CorpusFileList.options = {
  item: `
    <tr>
      <td><span class="filename"></span></td>
      <td><span class="author"></span></td>
      <td><span class="title"></span></td>
      <td><span class="publishing_year"></span></td>
      <td class="right-align">
        <a class="action-button btn-floating red tooltipped waves-effect waves-light" data-action="delete" data-position="top" data-tooltip="Delete"><i class="material-icons">delete</i></a>
        <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="download" data-position="top" data-tooltip="View"><i class="material-icons">file_download</i></a>
        <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="view" data-position="top" data-tooltip="View"><i class="material-icons">send</i></a>
      </td>
    </tr>
  `.trim(),
  valueNames: [{data: ['id']}, 'author', 'filename', 'publishing_year', 'title']
};
