class CorpusList extends RessourceList {
  constructor(listElement, options = {}) {
    super(listElement, {...CorpusList.options, ...options});
  }

  init(user) {
    super._init(user.corpora);
  }

  onclick(event) {
    let corpusElement = event.target.closest('tr[data-id]');
    if (corpusElement === null) {throw 'Could not locate corpus element';}
    let corpusId = corpusElement.dataset.id;
    let actionButtonElement = event.target.closest('.action-button[data-action]');
    let action = actionButtonElement === null ? 'view' : actionButtonElement.dataset.action;
    switch (action) {
      case 'delete':
        let deleteModalHTML = `
          <div class="modal">
            <div class="modal-content">
              <h4>Confirm corpus deletion</h4>
              <p>Do you really want to delete the corpus <b>${app.users[this.userId].corpora[corpusId].title}</b>? All files will be permanently deleted!</p>
            </div>
            <div class="modal-footer">
              <a href="#!" class="btn modal-close waves-effect waves-light">Cancel</a>
              <a class="btn modal-close red waves-effect waves-light" href="/corpora/${corpusId}/delete"><i class="material-icons left">delete</i>Delete</a>
            </div>
          </div>
        `.trim();
        let deleteModalParentElement = document.querySelector('#modals');
        deleteModalParentElement.insertAdjacentHTML('beforeend', deleteModalHTML);
        let deleteModalElement = deleteModalParentElement.lastChild;
        let deleteModal = M.Modal.init(deleteModalElement, {onCloseEnd: () => {deleteModal.destroy(); deleteModalElement.remove();}});
        deleteModal.open();
        break;
      case 'view':
        window.location.href = `/corpora/${corpusId}`;
        break;
      default:
        break;
    }
  }

  usersPatchHandler(patch) {
    let re = new RegExp(`^/users/${this.userId}/corpora/([A-Za-z0-9]*)`);
    let filteredPatch = patch.filter(operation => re.test(operation.path));
    for (let operation of filteredPatch) {
      switch(operation.op) {
        case 'add':
          re = new RegExp(`^/users/${this.userId}/corpora/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        case 'remove':
          re = new RegExp(`^/users/${this.userId}/corpora/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {
            let [match, corpusId] = operation.path.match(re);
            this.remove(corpusId);
          }
          break;
        case 'replace':
          re = new RegExp(`^/users/${this.userId}/corpora/([A-Za-z0-9]*)/(status|description|title)$`);
          if (re.test(operation.path)) {
            let [match, corpusId, valueName] = operation.path.match(re);
            this.replace(corpusId, valueName, operation.value);
          }
          break;
        default:
          break;
      }
    }
  }

  preprocessRessource(corpus) {
    return {
      id: corpus.id,
      creationDate: corpus.creation_date,
      description: corpus.description,
      status: corpus.status,
      title: corpus.title
    };
  }
}
CorpusList.options = {
  item: `
    <tr>
      <td><a class="btn-floating disabled"><i class="material-icons service-color darken" data-service="corpus-analysis">book</i></a></td>
      <td><b class="title"></b><br><i class="description"></i></td>
      <td><span class="badge new status status-color status-text" data-badge-caption=""></span></td>
      <td class="right-align">
        <a class="action-button btn-floating red tooltipped waves-effect waves-light" data-action="delete" data-position="top" data-tooltip="Delete"><i class="material-icons">delete</i></a>
        <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="view" data-position="top" data-tooltip="View"><i class="material-icons">send</i></a>
      </td>
    </tr>
  `.trim(),
  valueNames: [{data: ['id']}, {name: 'status', attr: 'data-status'}, 'description', 'title']
};
