class JobResultList extends RessourceList {
  constructor(listElement, options = {}) {
    super(listElement, {...JobResultList.options, ...options});
    this.jobId = listElement.dataset.jobId;
  }

  init(user) {
    super._init(user.jobs[this.jobId].results);
  }

  onclick(event) {
    let jobResultElement = event.target.closest('tr[data-id]');
    if (jobResultElement === null) {return;}
    let jobResultId = jobResultElement.dataset.id;
    let actionButtonElement = event.target.closest('.action-button[data-action]');
    if (actionButtonElement === null) {return;}
    let action = actionButtonElement.dataset.action;
    switch (action) {
      case 'download':
        window.location.href = `/jobs/${this.jobId}/results/${jobResultId}`;
        break;
      default:
        break;
    }
  }

  usersPatchHandler(patch) {
    let re = new RegExp(`^/users/${this.userId}/jobs/${this.jobId}/results/([A-Za-z0-9]*)`);
    let filteredPatch = patch.filter(operation => re.test(operation.path));
    for (let operation of filteredPatch) {
      switch(operation.op) {
        case 'add':
          re = new RegExp(`^/users/${this.userId}/jobs/${this.jobId}/results/([A-Za-z0-9]*)$`);
          if (re.test(operation.path)) {this.add(operation.value);}
          break;
        default:
          break;
      }
    }
  }

  preprocessRessource(jobResult) {
    let description;
    if (jobResult.filename.endsWith('.pdf.zip')) {
      description = 'PDF files with text layer';
    } else if (jobResult.filename.endsWith('.txt.zip')) {
      description = 'Raw text files';
    } else if (jobResult.filename.endsWith('.vrt.zip')) {
      description = 'VRT compliant files including the NLP data';
    } else if (jobResult.filename.endsWith('.xml.zip')) {
      description = 'TEI compliant files';
    } else if (jobResult.filename.endsWith('.poco.zip')) {
      description = 'HOCR and image files for post correction (PoCo)';
    } else {
      description = 'All result files created during this job';
    }
    return {
      id: jobResult.id,
      creationDate: jobResult.creation_date,
      description: description,
      filename: jobResult.filename
    };
  }
}
JobResultList.options = {
  item: `
    <tr>
      <td><span class="description"></span></td>
      <td><span class="filename"></span></td>
      <td class="right-align">
        <a class="action-button btn-floating tooltipped waves-effect waves-light" data-action="download" data-position="top" data-tooltip="View"><i class="material-icons">file_download</i></a>
      </td>
    </tr>
  `.trim(),
  valueNames: [{data: ['id']}, 'description', 'filename']
};
